"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var BigService = /** @class */ (function () {
    function BigService(radix) {
        if (radix === void 0) { radix = 16; }
        this.radix = radix;
    }
    // 去除开头的0
    BigService.prototype.ltrimZero = function (str) {
        if (!str) {
            return '';
        }
        var res = str.match(/^0*(\w+)/);
        return res ? res[1] : '';
    };
    // 大数加
    BigService.prototype.add = function (a, b) {
        a = a.toString();
        b = b.toString();
        if (a.indexOf('-') >= 0 && b.indexOf('-') < 0) {
            a = a.substring(1);
            return this.subtract(b, a);
        }
        else if (a.indexOf('-') < 0 && b.indexOf('-') >= 0) {
            b = b.substring(1);
            return this.subtract(a, b);
        }
        else if (a.indexOf('-') >= 0 && b.indexOf('-') >= 0) {
            a = a.substring(1);
            b = b.substring(1);
            return '-' + this.addWithoutSign(a, b);
        }
        else {
            return this.addWithoutSign(a, b);
        }
    };
    BigService.prototype.addWithoutSign = function (a, b) {
        var i;
        var j;
        var str1;
        var str2;
        var len1;
        var len2;
        var result = [];
        var output = [];
        str1 = a.toString().split('').reverse();
        str2 = b.toString().split('').reverse();
        len1 = str1.length;
        len2 = str2.length;
        for (i = 0; i < len1; i++) {
            for (j = 0; j <= len2; j++) {
                result[i + j] = 0;
                output[i + j] = result[i + j].toString(this.radix);
            }
        }
        for (i = 0; i < len1; i++) {
            for (j = 0; j < len2; j++) {
                result[i + j] += parseInt(str1[i], this.radix) * parseInt(str2[j], this.radix);
                output[i + j] = result[i + j].toString(this.radix);
            }
        }
        var n = result.length;
        for (var k = 0; k < n; k++) {
            var temp = result[k];
            if (temp >= this.radix) {
                result[k] = temp % this.radix;
                result[k + 1] += Math.floor(temp / this.radix);
                output[k] = result[k].toString(this.radix);
                output[k + 1] = result[k + 1].toString(this.radix);
            }
        }
        return this.ltrimZero(output.reverse().join('')) || '0';
    };
    // 大数乘
    BigService.prototype.multiply = function (a, b) {
        a = a.toString();
        b = b.toString();
        if (a.indexOf('-') >= 0 && b.indexOf('-') < 0) {
            a = a.substring(1);
            return '-' + this.multiplyWithoutSign(a, b);
        }
        else if (a.indexOf('-') < 0 && b.indexOf('-') >= 0) {
            b = b.substring(1);
            return '-' + this.multiplyWithoutSign(a, b);
        }
        else if (a.indexOf('-') >= 0 && b.indexOf('-') >= 0) {
            a = a.substring(1);
            b = b.substring(1);
            return this.multiplyWithoutSign(a, b);
        }
        else {
            return this.multiplyWithoutSign(a, b);
        }
    };
    BigService.prototype.multiplyWithoutSign = function (a, b) {
        var i;
        var j;
        var str1;
        var str2;
        var len1;
        var len2;
        var result = [];
        var output = [];
        str1 = a.toString().split('').reverse();
        str2 = b.toString().split('').reverse();
        len1 = str1.length;
        len2 = str2.length;
        for (i = 0; i < len1; i++) {
            for (j = 0; j <= len2; j++) {
                result[i + j] = 0;
                output[i + j] = result[i + j].toString(this.radix);
            }
        }
        for (i = 0; i < len1; i++) {
            for (j = 0; j < len2; j++) {
                result[i + j] += parseInt(str1[i], this.radix) * parseInt(str2[j], this.radix);
                output[i + j] = result[i + j].toString(this.radix);
            }
        }
        var n = result.length;
        for (var k = 0; k < n; k++) {
            // @ts-ignore
            var temp = result[k];
            if (temp >= this.radix) {
                result[k] = temp % this.radix;
                result[k + 1] += Math.floor(temp / this.radix);
                output[k] = result[k].toString(this.radix);
                output[k + 1] = result[k + 1].toString(this.radix);
            }
        }
        return this.ltrimZero(output.reverse().join('')) || '0';
    };
    // 大数减
    BigService.prototype.subtract = function (a, b) {
        a = a.toString();
        b = b.toString();
        if (a.indexOf('-') >= 0 && b.indexOf('-') < 0) {
            a = a.substring(1);
            return '-' + this.addWithoutSign(a, b);
        }
        else if (a.indexOf('-') < 0 && b.indexOf('-') >= 0) {
            b = b.substring(1);
            return this.addWithoutSign(a, b);
        }
        else if (a.indexOf('-') >= 0 && b.indexOf('-') >= 0) {
            a = a.substring(1);
            b = b.substring(1);
            return this.subtractWithoutSign(b, a);
        }
        else {
            return this.subtractWithoutSign(a, b);
        }
    };
    // 大数无符号减
    BigService.prototype.subtractWithoutSign = function (a, b) {
        a = a.toString();
        b = b.toString();
        var output = [];
        var minusOne = 0;
        a = this.ltrimZero(a);
        b = this.ltrimZero(b);
        // 补0对齐
        while (a.length < b.length) {
            a = '0' + a;
        }
        while (b.length < a.length) {
            b = '0' + b;
        }
        // 从后面位数向前相减
        for (var i = a.length - 1; i >= 0; i--) {
            var c1 = parseInt(a.charAt(i), this.radix);
            var c2 = parseInt(b.charAt(i), this.radix);
            // 如果当前位数无须借位
            if (c1 - minusOne >= c2) {
                output.unshift((c1 - c2 - minusOne).toString(this.radix));
                minusOne = 0;
            }
            else {
                output.unshift((c1 + this.radix - c2 - minusOne).toString(this.radix));
                minusOne = 1;
            }
        }
        // 如果最高位仍然要借位
        if (minusOne) {
            return '-' + this.ltrimZero(this.subtract(b, a));
        }
        return this.ltrimZero(output.join(''));
    };
    // 大数除法(只支持整数除法)
    BigService.prototype.divide = function (a, b) {
        a = a.toString();
        b = b.toString();
        if (a.indexOf('-') >= 0 && b.indexOf('-') < 0) {
            return '-' + this.divideWithoutSign(a.substring(1), b);
        }
        else if (a.indexOf('-') < 0 && b.indexOf('-') >= 0) {
            return '-' + this.divideWithoutSign(a, b.substring(1));
        }
        else if (a.indexOf('-') >= 0 && b.indexOf('-') >= 0) {
            return this.divideWithoutSign(a.substring(1), b.substring(1));
        }
        else {
            return this.divideWithoutSign(a, b);
        }
    };
    // 无符号除法,返回商
    BigService.prototype.divideWithoutSign = function (a, b) {
        var c;
        a = a.toString();
        b = b.toString();
        if (this.compare(a, b) < 0) {
            return '0';
        }
        else if (this.compare(a, b) === 0) {
            return '1';
        }
        var alen = a.length;
        var blen = b.length;
        var times = alen - blen + 1;
        var result = [];
        for (c = 0; c < times; c++) {
            var tempb = b;
            result[c] = 0;
            // 补0
            while (tempb.length < alen - c) {
                tempb = tempb + '0';
            }
            while (this.compare(a, tempb) >= 0) {
                result[c]++;
                a = this.subtract(a, tempb);
            }
        }
        for (c = 0; c < result.length; c++) {
            result[c] = result[c].toString(this.radix);
        }
        return this.ltrimZero(result.join(''));
    };
    // 取余(a可正可负，b只能为正数),最后的结果恒为非负数
    BigService.prototype.mod = function (a, b) {
        a = a.toString();
        b = b.toString();
        if (b.indexOf('-') !== -1) {
            return 'b只能为正数';
        }
        // a - (a/b)*b
        var output = this.divide(a, b);
        output = this.multiply(output, b);
        output = this.subtract(a, output);
        if (output.indexOf('-') !== -1) {
            output = this.add(output, b);
        }
        return output;
    };
    // 比较函数(1:a大，±0：相等，-1,：b大)
    BigService.prototype.compare = function (a, b) {
        a = a.toString();
        b = b.toString();
        var sign = 1;
        a = a.toString();
        b = b.toString();
        if (a.indexOf('-') >= 0 && b.indexOf('-') < 0) {
            return -1;
        }
        else if (a.indexOf('-') < 0 && b.indexOf('-') >= 0) {
            return 1;
        }
        else if (a.indexOf('-') >= 0 && b.indexOf('-') >= 0) {
            sign = -1;
            a = a.substring(1);
            b = b.substring(1);
        }
        a = a.replace(/^0+/, '');
        b = b.replace(/^0+/, '');
        var flag;
        if (a.length < b.length) {
            flag = -1;
        }
        else if (a.length > b.length) {
            flag = 1;
        }
        else {
            flag = 0;
        }
        if (flag === 0) {
            var aArr = a.split('');
            var bArr = b.split('');
            for (var i = 0; i <= aArr.length; i++) {
                if (aArr[i] > bArr[i]) {
                    flag = 1;
                    break;
                }
                else if (aArr[i] < bArr[i]) {
                    flag = -1;
                    break;
                }
            }
        }
        return sign * flag;
    };
    // 大数随机数(生成0-n之间的随机数)
    BigService.prototype.randomNum = function (n) {
        n = n.toString();
        var flag = 10000000000000000;
        var randomWithoutRadix = Math.random();
        var random = randomWithoutRadix.toString(this.radix);
        var start = random.indexOf('.') + 1;
        var randomOutput = random.substring(start, start + random.length);
        return this.divide(this.multiply(randomOutput, n), flag.toString(this.radix));
    };
    return BigService;
}());
exports.default = BigService;
