"use strict";
// import {ab2hex, toBlueBufferData} from "../utils/util";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlueDevice = exports.prefix = exports.Suppliers = exports.versions = void 0;
var util_1 = require("../utils/util");
var config_1 = __importDefault(require("../config"));
var regIds = [/^G([\d]{15})/, /^EGS-([\d]{6})/];
exports.versions = {
    0: 'ZL1',
    1: 'ZL2',
    2: 'TS',
    x: 'YX'
};
exports.Suppliers = ['ZL', 'YX', 'TS'];
exports.prefix = '_zl_';
var BlueDevice = /** @class */ (function () {
    function BlueDevice(device) {
        this.name = '';
        this.blueId = '';
        this.deviceId = '';
        this.serviceId = '';
        this.characteristicId = '';
        this.localName = '';
        this.RSSI = 0;
        this.ecsNo = '';
        this.advertisString = '';
        this.supplier = 'ZL';
        this.auth = false;
        this.write = false;
        this.connected = false;
        this.isValid = true;
        this.process = false;
        this.processStartTime = 0;
        this.version = exports.versions[1];
        this.state = 0;
        for (var _i = 0, _a = Object.keys(device); _i < _a.length; _i++) {
            var k = _a[_i];
            // @ts-ignore
            this[k] = device[k];
        }
    }
    // get() {
    //     Object.getOwnPropertyNames(this).forEach();
    // }
    BlueDevice.prototype.setSupplier = function (s) {
        this.supplier = s;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setName = function (n) {
        this.name = n;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setState = function (s) {
        this.state = s;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setAdvertisString = function (a) {
        this.advertisString = a;
    };
    BlueDevice.prototype.setVersion = function (v) {
        this.version = v;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setIsValid = function (i) {
        this.isValid = i;
    };
    BlueDevice.prototype.setConnected = function (c) {
        this.connected = c;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setWrite = function (w) {
        this.write = w;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setAuth = function (a) {
        this.auth = a;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setBlueId = function (bId) {
        this.blueId = bId;
        this.ecsNo = bId.length === 15 ? bId : '';
        // wx.setStorageSync('blue:d', this)
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setServiceId = function (serviceId) {
        this.serviceId = serviceId;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setCharacteristicId = function (characteristicId) {
        this.characteristicId = characteristicId;
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.setProcess = function (process) {
        this.process = process;
        if (this.process) {
            this.processStartTime = Math.ceil((new Date()).valueOf() / 1000);
        }
        wx.setStorageSync(config_1.default.storagePrefix ? config_1.default.storagePrefix + ':blue:d' : 'blue:d', this);
    };
    BlueDevice.prototype.getProcess = function () {
        return this.process && (Math.ceil((new Date()).valueOf() / 1000) - this.processStartTime < 10);
    };
    BlueDevice.prototype.decrypt = function () {
        if (!this.name && !this.localName) {
            this.setIsValid(false);
            return;
        }
        var temp = this.parseName(this.name || this.localName);
        if (!temp) {
            this.setIsValid(false);
            return;
        }
        // FIXME::ts
        console.log('====设备ID==', this.blueId);
        this.setIsValid(true);
        this.parseAdvertisData();
    };
    BlueDevice.prototype.encrypt = function () {
        return true;
    };
    BlueDevice.prototype.parseName = function (name) {
        var len = regIds.length;
        for (var i = 0; i < len; i++) {
            if (regIds[i].test(name)) {
                // 新版
                if (name.length > 16) {
                    var sup = Number(name.substring(16, 17));
                    this.setSupplier(exports.Suppliers[sup] || exports.Suppliers[0]);
                    if (this.supplier === exports.Suppliers[0]) {
                        this.setAdvertisString('01');
                    }
                    // 老版，只有无锡和深圳存在
                }
                else {
                    this.setSupplier(exports.Suppliers[i]);
                }
                // @ts-ignore
                this.setBlueId(name.match(regIds[i])[1]);
                return true;
            }
        }
        return false;
    };
    BlueDevice.prototype.parseAdvertisData = function () {
        var advertisString = '';
        if (this.advertisData instanceof ArrayBuffer) {
            advertisString = (0, util_1.ab2hex)(this.advertisData);
        }
        // FIXME::ts
        console.log('====供应商', this.supplier);
        console.log('====广播值', advertisString);
        switch (this.supplier) {
            case 'YX': {
                this.setVersion(exports.versions.x);
                break;
            }
            case 'TS': {
                this.setAdvertisString(advertisString);
                this.setVersion(exports.versions[2]);
                break;
            }
            default: {
                if (!advertisString) {
                    if (this.advertisString === '01') {
                        this.setAdvertisString('01');
                        this.setVersion(exports.versions[1]);
                    }
                    else {
                        this.setVersion(exports.versions[0]);
                        this.setAdvertisString('0');
                    }
                }
                else {
                    this.setAdvertisString(advertisString);
                    this.setVersion(exports.versions[parseInt(advertisString.substring(0, 2), 10)] || exports.versions[1]);
                }
            }
        }
        // if (! (this.advertisData instanceof ArrayBuffer)) {
        //     this.setAdvertisString('0')
        //     this.setVersion(versions[0])
        //     return;
        // }
        // advertisString = ab2hex(this.advertisData);
        // this.setAdvertisString(advertisString)
        // this.setVersion(versions[parseInt(advertisString.substring(0, 2), 10) as keyof typeof versions] || versions[0])
    };
    return BlueDevice;
}());
exports.BlueDevice = BlueDevice;
