import { BlueDevice } from "./BlueDevice";
import { NONE_METHOD, AUTH, ON, OFF, PICK_FACE, DEL_FACE, CLEAR_FACE, GET_FACE, PICK_FINGER, DEL_FINGER, CLEAR_FINGER, GET_FINGER, FETCH_FINGER, ADD_DEVICE, DEL_DEVICE, GET_DEVICES, GET_DEVICE_MAC, GET_DEVICES_ONLINE, D_PICK_FINGER, D_DEL_FINGER, D_CLEAR_FINGER, D_GET_FINGER, HANDLE_DATA } from "./Methods";
export declare type Config = {
    abnormalCloseOnReSearch?: boolean;
    storagePrefix?: string;
    showToast?: boolean;
    isContinue?: boolean;
    [key: string]: any;
};
export declare type Message = {
    data: string;
    down?: boolean;
    time: string;
};
export declare type Methods = ON | OFF | PICK_FACE | DEL_FACE | CLEAR_FACE | GET_FACE | PICK_FINGER | DEL_FINGER | CLEAR_FINGER | GET_FINGER | FETCH_FINGER | HANDLE_DATA | ADD_DEVICE | DEL_DEVICE | GET_DEVICE_MAC | GET_DEVICES | D_PICK_FINGER | D_DEL_FINGER | D_CLEAR_FINGER | D_GET_FINGER | GET_DEVICES_ONLINE | "" | NONE_METHOD;
export declare type NotifyType = AUTH | Methods;
export declare type ProcessDetail = {
    type: NotifyType | string;
    status: boolean;
    process: 0 | 1 | 2;
    message?: string;
};
export declare type NotifyMessage = {
    type: NotifyType;
    status: boolean;
    process?: 0 | 1 | 2;
    detail?: {
        id: number;
        name: string;
        [key: string]: any;
    };
    message?: string;
    data?: {
        id: string;
        [key: string]: any;
    }[];
};
export declare type FailFn = (errorCode: number, error: string | object) => void;
export declare type CommonFn = (device?: BlueDevice, type?: NotifyType) => void;
export declare type OriginNotifyFn = (device: BlueDevice, message: Message) => void;
export declare type NotifyFn = (device: BlueDevice, message: NotifyMessage) => void;
export declare type ProcessFn = (device: BlueDevice, process: boolean, detail: ProcessDetail) => void;
export declare type Cb = (res: any) => void;
export declare type ShowToastFn = (title: string, type: 'success' | 'error' | 'none', device?: BlueDevice, error?: string | object) => void;
export declare type Hooks = {
    discoveried?: CommonFn;
    beforeConnect?: CommonFn;
    connected?: CommonFn;
    authed?: CommonFn;
    beforeSend?: CommonFn;
    sent?: CommonFn;
    beforeUpdate?: CommonFn;
    updated?: CommonFn;
    beforeClose?: CommonFn;
    closed?: CommonFn;
    showToast?: ShowToastFn;
    fail?: FailFn;
    originNotify?: OriginNotifyFn;
    process?: ProcessFn;
    notify?: NotifyFn;
};
export declare type HandleType = 'down' | 'password' | 'error' | 'none';
export declare type PickDetail = {
    name: string;
    expire?: string;
    mobile?: string;
    ecsNo?: string;
    company?: string;
};
export declare type SetWheelPerimeterDetail = {
    wheelPerimeter: number;
    deviceId: string;
    macAddress?: string;
    id?: string;
};
export declare type FindOrDeleteDetail = {
    id: number;
    ecsNo?: string;
    name?: string;
};
export declare type PickDetailType = PickDetail | FindOrDeleteDetail | string | undefined | null;
export declare type NextCbType = 'noSend' | 'send';
export declare class BlueService {
    protected device?: BlueDevice;
    protected devices: BlueDevice[];
    protected config: Config;
    protected discoveryStarted: boolean;
    protected isLastPackage: boolean;
    protected discoveried?: CommonFn;
    protected beforeConnect?: CommonFn;
    protected connected?: CommonFn;
    protected authed?: CommonFn;
    protected beforeSend?: CommonFn;
    protected sent?: CommonFn;
    protected beforeUpdate?: CommonFn;
    protected updated?: CommonFn;
    protected beforeClose?: CommonFn;
    protected closed?: CommonFn;
    protected showToast?: ShowToastFn;
    protected fail?: FailFn;
    protected originNotify?: OriginNotifyFn;
    protected notify?: NotifyFn;
    protected process?: ProcessFn;
    constructor(hooks?: Hooks, config?: Config);
    protected setDevice(device: BlueDevice): void;
    getDevice(): BlueDevice | undefined;
    protected setConfig(config: Config): void;
    protected pushDevices(device: BlueDevice): void;
    getDevices(): BlueDevice[];
    protected setDiscoveryStarted(state: boolean): void;
    protected getDiscoveryStarted(): boolean;
    protected setIsLastPackage(isLast: boolean): void;
    protected toast(title: string, icon: 'success' | 'error' | 'none', error?: object | string): void;
    searchBlue(): void;
    listenBleChange(): void;
    openBluetoothAdapter(): void;
    protected getBluetoothAdapterState(): void;
    startBluetoothDevicesDiscovery(): void;
    onBluetoothDeviceFound(): void;
    getBluetoothDevices(): void;
    stopBluetoothDevicesDiscovery(s?: Cb, f?: Cb): void;
    closeConnection(isSearch?: boolean, isForce?: boolean): void;
    closeBluetoothAdapter(isSearch?: boolean): void;
    connect(device: BlueDevice): void;
    getBLEDeviceServices(): void;
    getBLEDeviceCharacteristics(): void;
    handleResponse(data: string): void;
    writeToBlue(buffer: ArrayBuffer, type?: NotifyType, str?: string): void;
    send(str: string, type?: NotifyType): void;
    sendChunk(str: string, type?: NotifyType, len?: number): void;
    handleNextCb(str: string, type?: NextCbType, detail?: NotifyMessage): void;
    handleProcessCb(process: boolean, detail: ProcessDetail): void;
    handleFailCb(code: number, message: string): void;
    handle(method?: Methods, detail?: any): void;
}
