"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlueService = void 0;
/// <reference types="typescript" />
var util_1 = require("../utils/util");
var BlueDevice_1 = require("./BlueDevice");
var CommonVersion_1 = require("./versions/CommonVersion");
var ZL1Version_1 = require("./versions/ZL1Version");
var YXVersion_1 = require("./versions/YXVersion");
var config_1 = __importDefault(require("../config"));
var error_1 = __importDefault(require("../error"));
var dayjs_1 = __importDefault(require("dayjs"));
var Methods_1 = __importDefault(require("./Methods"));
var versionToService = {
    'ZL1': ZL1Version_1.ZL1Version,
    'ZL2': CommonVersion_1.CommonVersion,
    'YX': YXVersion_1.YXVersion,
    'TS': CommonVersion_1.CommonVersion,
};
var SERVICE_UUIDS = [
    "0000FF00-0000-1000-8000-00805F9B34FB",
    "0000FFF0-0000-1000-8000-00805F9B34FB",
    "0000FF10-0000-1000-8000-00805F9B34FB", // 塔上
    // "0000FF00-0000-1000-8000-00805F9B34FB", // 深圳高机
];
var success = function (title) {
    wx.showToast({
        title: title,
        icon: 'success'
    });
};
var fail = function (title) {
    wx.showToast({
        title: title,
        icon: 'error'
    });
};
var BlueService = /** @class */ (function () {
    function BlueService(hooks, config) {
        if (hooks === void 0) { hooks = {}; }
        if (config === void 0) { config = {}; }
        var _this = this;
        // 已经连接的设备
        this.device = undefined;
        // 所有设备
        this.devices = [];
        this.config = {};
        this.discoveryStarted = false;
        this.isLastPackage = true;
        this.setConfig(config);
        Object.keys(hooks).forEach(function (method) {
            if (hooks[method]) {
                // @ts-ignore
                _this[method] = hooks[method];
            }
        });
    }
    BlueService.prototype.setDevice = function (device) {
        this.device = device;
    };
    BlueService.prototype.getDevice = function () {
        return this.device;
    };
    BlueService.prototype.setConfig = function (config) {
        this.config = __assign(__assign({}, config_1.default), config);
    };
    BlueService.prototype.pushDevices = function (device) {
        this.devices.push(device);
    };
    BlueService.prototype.getDevices = function () {
        return this.devices;
    };
    BlueService.prototype.setDiscoveryStarted = function (state) {
        this.discoveryStarted = state;
    };
    BlueService.prototype.getDiscoveryStarted = function () {
        return this.discoveryStarted;
    };
    BlueService.prototype.setIsLastPackage = function (isLast) {
        this.isLastPackage = isLast;
    };
    BlueService.prototype.toast = function (title, icon, error) {
        if (error === void 0) { error = ''; }
        if (!this.config.showToast) {
            return;
        }
        switch (icon) {
            case 'success': {
                if (this.showToast) {
                    this.showToast(title, 'success', this.device, '');
                }
                else {
                    success(title);
                }
                break;
            }
            case 'error': {
                if (this.showToast) {
                    this.showToast(title, 'error', this.device, error);
                }
                else {
                    fail(title);
                }
                break;
            }
            default: {
                console.info(title);
            }
        }
    };
    // 检索蓝牙
    BlueService.prototype.searchBlue = function () {
        this.closeConnection(true);
        // this.closeBluetoothAdapter();
        // this.openBluetoothAdapter();
    };
    // 监听蓝牙状态变化
    BlueService.prototype.listenBleChange = function () {
        var _this = this;
        wx.onBLEConnectionStateChange(function (res) {
            // 该方法回调中可以用于处理连接意外断开等异常情况
            // console.log(`device ${res.deviceId} state has changed, connected: ${res.connected}`)
            // FIXME::ts
            console.log("==== device ".concat(res.deviceId, " state has changed, connected: ").concat(res.connected));
            if (!res.connected) {
                // Toast.fail('蓝牙已断开');
                wx.removeStorageSync('yxdata'); // 清空数据
                wx.removeStorageSync('mtu');
            }
            if ((!res.connected) && _this.device && (_this.device.deviceId === res.deviceId)) {
                if (_this.device.setConnected) {
                    _this.device.setConnected(false);
                    _this.device.setAuth(false);
                    _this.device.setWrite(false);
                    _this.device.setProcess(false);
                    // 通知到上游
                    if (_this.closed) {
                        _this.closed(_this.device);
                    }
                }
                if (_this.config.abnormalCloseOnReSearch) {
                    console.debug('蓝牙断开，尝试search');
                    _this.searchBlue();
                }
            }
        });
    };
    // 开启蓝牙适配
    BlueService.prototype.openBluetoothAdapter = function () {
        var _this = this;
        var retryCount = 0;
        var fn = function () {
            retryCount++;
            wx.openBluetoothAdapter({
                success: function (res) {
                    wx.onBluetoothAdapterStateChange(function (r) {
                        console.log('onBluetoothAdapterStateChange', r);
                    });
                    console.info('openBluetoothAdapter success');
                    _this.startBluetoothDevicesDiscovery();
                },
                fail: function (res) {
                    if (retryCount <= 4) {
                        fn();
                    }
                    else {
                        if (_this.fail) {
                            _this.fail(error_1.default.openAdapterError, res);
                        }
                        else {
                            _this.toast('请打开蓝牙', 'error', res);
                        }
                    }
                    // if (res.errCode === 10001) {
                    //     wx.onBluetoothAdapterStateChange((r) => {
                    //         console.log('onBluetoothAdapterStateChange', r)
                    //         if (r.available) {
                    //             this.startBluetoothDevicesDiscovery()
                    //         }
                    //     })
                    // }
                    // else {
                    //     if (this.fail) {
                    //         this.fail(errorCode.openAdapterError, res)
                    //     } else {
                    //         this.toast('请打开蓝牙', 'error', res)
                    //     }
                    // }
                }
            });
        };
        fn();
    };
    BlueService.prototype.getBluetoothAdapterState = function () {
        var _this = this;
        wx.getBluetoothAdapterState({
            success: function (res) {
                // console.log('getBluetoothAdapterState', res)
                if (res.discovering) {
                    _this.getBluetoothDevices();
                    _this.onBluetoothDeviceFound();
                }
                else if (res.available) {
                    _this.startBluetoothDevicesDiscovery();
                }
            }
        });
    };
    BlueService.prototype.startBluetoothDevicesDiscovery = function () {
        var _this = this;
        // 暂时不考虑，主要是 onBluetoothAdapterStateChange 也会监听
        if (this.discoveryStarted) {
            // if (this.fail) {
            //     this.fail(errorCode.discoveryStartedError, '蓝牙检索已经打开')
            // } else {
            //     this.toast('蓝牙检索已经打开', 'error')
            // }
            // this.toast('蓝牙检索已经打开', 'error')
            console.info('蓝牙检索已经打开');
            return;
        }
        // 如果已经连接也不开始
        if (this.device && this.device.connected) {
            return;
        }
        wx.startBluetoothDevicesDiscovery({
            allowDuplicatesKey: true,
            success: function (res) {
                _this.setDiscoveryStarted(true);
                // console.log('startBluetoothDevicesDiscovery success', res)
                _this.getBluetoothDevices();
                _this.onBluetoothDeviceFound();
            },
            fail: function (res) {
                _this.setDiscoveryStarted(false);
                if (_this.fail) {
                    _this.fail(error_1.default.discoveryError, res);
                }
                else {
                    _this.toast('蓝牙检索打开失败', 'error');
                }
            }
        });
    };
    BlueService.prototype.onBluetoothDeviceFound = function () {
        var _this = this;
        wx.onBluetoothDeviceFound(function (res) {
            // console.log('found')
            res.devices.forEach(function (device) {
                var item = new BlueDevice_1.BlueDevice(device);
                item.decrypt();
                if (!item.isValid) {
                    return;
                }
                // TODO::发现可用设备列表
                // BlueService.devices.push(item)
                _this.pushDevices(item);
                if (_this.discoveried) {
                    _this.discoveried(item);
                }
            });
        });
    };
    BlueService.prototype.getBluetoothDevices = function () {
        var _this = this;
        wx.getBluetoothDevices({
            success: function (res) {
                // console.info('getBluetoothDevices-old')
                res.devices.forEach(function (device) {
                    var item = new BlueDevice_1.BlueDevice(device);
                    item.decrypt();
                    if (!item.isValid) {
                        return;
                    }
                    // 发现可用设备列表
                    // BlueService.devices.push(item)
                    _this.pushDevices(item);
                    if (_this.discoveried) {
                        _this.discoveried(item);
                    }
                });
            },
            fail: function (res) {
                console.error('getBluetoothDevices', { res: res });
                if (res.errno && res.errno === 1500101) {
                    console.log('尝试重启');
                    _this.setDiscoveryStarted(false);
                    _this.openBluetoothAdapter();
                }
            }
        });
    };
    BlueService.prototype.stopBluetoothDevicesDiscovery = function (s, f) {
        var _this = this;
        this.setDiscoveryStarted(false);
        wx.stopBluetoothDevicesDiscovery().then(function (r) {
            if (!s) {
                console.log({ r: r });
                return;
            }
            s(r);
        }).catch(function (res) {
            if (f) {
                f(res);
                return;
            }
            if (_this.fail) {
                _this.fail(error_1.default.stopDiscoveryError, res);
            }
            else {
                _this.toast('停止蓝牙检索失败', 'error');
            }
        });
    };
    // 断开连接
    BlueService.prototype.closeConnection = function (isSearch, isForce) {
        var _this = this;
        if (isSearch === void 0) { isSearch = false; }
        if (isForce === void 0) { isForce = false; }
        if (isSearch) {
            this.setDiscoveryStarted(false);
        }
        if (!this.device || !this.device.connected) {
            if (isSearch) {
                this.openBluetoothAdapter();
            }
            console.info('设备未连接，无需断开', { isSearch: isSearch });
            if (!isSearch) {
                if (this.fail) {
                    this.fail(error_1.default.cantCloseError, '设备未连接，无需断开');
                }
                else {
                    this.toast('设备未连接，无需断开', 'error');
                }
            }
            if (!isForce) {
                return;
            }
        }
        if (this.beforeClose) {
            this.beforeClose(this.device);
        }
        if (this.device) {
            wx.closeBLEConnection({
                deviceId: this.device.deviceId || ''
            }).then(function () {
                _this.closeBluetoothAdapter(isSearch);
                wx.removeStorageSync('yxdata'); // 清空数据
                wx.removeStorageSync('mtu');
                if (_this.device && _this.device.setConnected) {
                    _this.device.setConnected(false);
                    _this.device.setWrite(false);
                    _this.device.setAuth(false);
                    if (_this.closed) {
                        _this.closed(_this.device);
                    }
                }
            }).catch(function (e) {
                console.error(e);
                if (_this.fail) {
                    _this.fail(error_1.default.closeError, e);
                }
                else {
                    _this.toast('关闭连接错误', 'error');
                }
            });
        }
    };
    // 关闭蓝牙适配
    BlueService.prototype.closeBluetoothAdapter = function (isSearch) {
        var _this = this;
        if (isSearch === void 0) { isSearch = false; }
        // if (this.device && this.device.setConnected) this.device.setConnected(false)
        // this.discoveryStarted = false
        this.setDiscoveryStarted(false);
        wx.closeBluetoothAdapter().then(function (r) {
            if (isSearch) {
                _this.openBluetoothAdapter();
            }
            console.log({ r: r });
        }).catch(function (e) {
            console.error('关闭蓝牙适配器失败', e);
            if (isSearch) {
                _this.openBluetoothAdapter();
            }
            if (_this.fail) {
                _this.fail(error_1.default.closeAdapterError, e);
            }
            else {
                _this.toast('关闭蓝牙适配器失败', 'error');
            }
        });
    };
    BlueService.prototype.connect = function (device) {
        var _this = this;
        if (this.device && this.device.connected) {
            if (this.fail) {
                this.fail(error_1.default.alreadyError, '设备已经连接或者已存在连接设备');
            }
            else {
                this.toast('设备已经连接或者已存在连接设备', 'error');
            }
            return;
        }
        this.listenBleChange();
        var retryCount = 0;
        var connectBluetooth = function () {
            retryCount++;
            // noinspection JSUnresolvedVariable
            wx.createBLEConnection({
                deviceId: device.deviceId,
                success: function () {
                    // noinspection JSUnresolvedVariable
                    wx.setBLEMTU({
                        deviceId: device.deviceId,
                        mtu: 37,
                        success: function (res) {
                            console.log({ mtuok: res });
                        },
                        fail: function (res) {
                            console.log('mtu设置失败', { mtuerr: res });
                        }
                    });
                    _this.setDevice(device);
                    if (!_this.device) {
                        if (_this.fail) {
                            _this.fail(error_1.default.connectingError, '蓝牙连接时出错');
                        }
                        else {
                            _this.toast('蓝牙连接时出错', 'error');
                        }
                        return;
                    }
                    _this.device.setConnected(true);
                    // TODO:: 原夏进行特殊处理
                    if (_this.device.version === 'YX') {
                        _this.device.setAuth(true);
                    }
                    if (_this.updated) {
                        _this.updated(_this.device);
                    }
                    if (_this.connected) {
                        _this.connected(_this.device);
                    }
                    _this.getBLEDeviceServices();
                    _this.stopBluetoothDevicesDiscovery();
                    _this.toast('蓝牙连接成功！', 'success');
                    // FIXME::ts
                    console.log('====蓝牙连接成功!', _this.device.version);
                },
                fail: function (res) {
                    if (res.errno === 1509007) {
                        console.log('已经连接先断开', retryCount);
                        _this.closeConnection(false, true);
                    }
                    else {
                        console.log('开始重新连接', retryCount);
                        if (retryCount <= 3) {
                            connectBluetooth();
                        }
                        else {
                            // fail('蓝牙连接失败！')
                            // this.toast('蓝牙连接失败！', 'error', res)
                            console.log({ res: res }, '蓝牙连接失败！');
                            if (_this.fail) {
                                // FIXME::ts
                                console.log('====蓝牙连接失败!', error_1.default.connectionError);
                                _this.fail(error_1.default.connectionError, '蓝牙连接失败！');
                            }
                            else {
                                _this.toast('蓝牙连接失败！', 'error');
                            }
                        }
                    }
                }
            });
        };
        connectBluetooth();
    };
    BlueService.prototype.getBLEDeviceServices = function () {
        var _this = this;
        if (!this.device || !this.device.deviceId) {
            if (this.fail) {
                this.fail(error_1.default.notFoundDeviceError, '未找到需要连接的设备');
            }
            else {
                this.toast('未找到需要连接的设备', 'error');
            }
            return;
        }
        wx.getBLEDeviceServices({
            deviceId: this.device.deviceId,
            success: function (res) {
                // console.log({s: res.services})
                var services = res.services.filter(function (item) {
                    return SERVICE_UUIDS.includes(item.uuid.toUpperCase());
                });
                // console.log({services})
                if (services.length < 1) {
                    if (_this.fail) {
                        _this.fail(error_1.default.notFoundServiceError, '未发现可用服务');
                    }
                    else {
                        _this.toast('未发现可用服务', 'error');
                    }
                    // BlueService.initBlue()
                    _this.searchBlue();
                    return;
                }
                if (_this.device && _this.device.setServiceId) {
                    _this.device.setServiceId(services[0].uuid);
                }
                _this.getBLEDeviceCharacteristics();
            }
        });
    };
    BlueService.prototype.getBLEDeviceCharacteristics = function () {
        var _this = this;
        if (!this.device || !this.device.deviceId || !this.device.serviceId) {
            if (this.fail) {
                this.fail(error_1.default.notFoundDeviceServiceError, '未找到需要连接的设备或服务');
            }
            else {
                this.toast('未找到需要连接的设备或服务', 'error');
            }
            return;
        }
        wx.getBLEDeviceCharacteristics({
            deviceId: this.device.deviceId,
            serviceId: this.device.serviceId,
            success: function (res) {
                console.log('getBLEDeviceCharacteristics success', res.characteristics);
                for (var _i = 0, _a = res.characteristics; _i < _a.length; _i++) {
                    var item = _a[_i];
                    // let item = res.characteristics[i]
                    if (!_this.device || !_this.device.deviceId || !_this.device.serviceId) {
                        if (_this.fail) {
                            _this.fail(error_1.default.notFoundDeviceServiceError, '未找到需要连接的设备或服务');
                        }
                        else {
                            _this.toast('未找到需要连接的设备或服务', 'error');
                        }
                        continue;
                    }
                    // FIXME::ts
                    console.log('====deviceId', _this.device.deviceId);
                    console.log('====serviceId', _this.device.serviceId);
                    console.log('====characteristicId', item.uuid);
                    if (item.properties.read) {
                        wx.readBLECharacteristicValue({
                            deviceId: _this.device.deviceId,
                            serviceId: _this.device.serviceId,
                            characteristicId: item.uuid,
                            // success (res) {
                            //     console.log('readBLECharacteristicValue:', res)
                            // }
                        });
                    }
                    if (item.properties.write) {
                        _this.device.setWrite(true);
                        _this.device.setCharacteristicId(item.uuid);
                    }
                    if (item.properties.notify || item.properties.indicate) {
                        // noinspection JSUnresolvedVariable
                        wx.notifyBLECharacteristicValueChange({
                            deviceId: _this.device.deviceId,
                            serviceId: _this.device.serviceId,
                            characteristicId: item.uuid,
                            state: true,
                            // success: () => {},
                            // fail: () => {}
                        });
                    }
                }
            },
            fail: function (res) {
                console.error('getBLEDeviceCharacteristics', res);
                // FIXME::ts
                console.log('====获取特征值失败');
                if (_this.fail) {
                    _this.fail(error_1.default.getCharacteristicsError, res);
                }
                else {
                    _this.toast('获取特征值失败', 'error');
                }
            }
        });
        // 操作之前先监听，保证第一时间获取数据
        wx.onBLECharacteristicValueChange(function (characteristic) {
            if (!_this.device) {
                if (_this.fail) {
                    _this.fail(error_1.default.notFoundDeviceError, '未找到需要连接的设备');
                }
                else {
                    _this.toast('未找到需要连接的设备', 'error');
                }
                return;
            }
            // FIXME::ts
            // console.log('===原始数据!', characteristic.value)
            var data = (0, util_1.ab2hex)(characteristic.value);
            // FIXME::ts
            // console.log('===处理数据!', data)
            // console.log('返回值', data)
            if (_this.config.originNotify && _this.originNotify) {
                console.log('up', data);
                _this.originNotify(_this.device, {
                    data: data,
                    down: false,
                    time: (0, dayjs_1.default)().format('YYYY-MM-DD hh:mm:ss')
                });
            }
            // TODO:: 处理数据
            _this.handleResponse(data);
        });
    };
    // 处理数据
    BlueService.prototype.handleResponse = function (data) {
        var _this = this;
        if (!this.device || !this.device.version) {
            if (this.fail) {
                this.fail(error_1.default.notFoundVersionError, '未找到蓝牙版本');
            }
            else {
                this.toast('未找到蓝牙版本', 'error');
            }
            return;
        }
        // 获取版本号
        if (this.device.version && versionToService[this.device.version]) {
            /**
             * @param {CommonVersion} v
             */
            var v = new (versionToService[this.device.version])({
                fail: function (code, message) {
                    _this.handleFailCb(code, message);
                },
                next: function (str, type, detail) {
                    if (type === void 0) { type = 'send'; }
                    if (detail === void 0) { detail = {
                        type: 'none',
                        status: false
                    }; }
                    _this.handleNextCb(str, type, detail);
                },
                process: function (process, detail) {
                    _this.handleProcessCb(process, detail);
                }
            }, this.config, this.device.version);
            v.handle('handleData', data);
        }
    };
    BlueService.prototype.writeToBlue = function (buffer, type, str) {
        var _this = this;
        if (!this.device || !this.device.deviceId || !this.device.serviceId || !this.device.characteristicId) {
            if (this.fail) {
                this.fail(error_1.default.notFoundAllError, '未找到需要连接的设备/服务/服务特征');
            }
            else {
                this.toast('未找到需要连接的设备/服务/服务特征', 'error');
            }
            return;
        }
        wx.writeBLECharacteristicValue({
            deviceId: this.device.deviceId,
            serviceId: this.device.serviceId,
            characteristicId: this.device.characteristicId,
            value: buffer,
            success: function () {
                // console.log("写入成功")
                if (!_this.device) {
                    if (_this.fail) {
                        _this.fail(error_1.default.notFoundDeviceError, '未找到需要连接的设备');
                    }
                    else {
                        _this.toast('未找到需要连接的设备', 'error');
                    }
                    return;
                }
                if (_this.config.originNotify && _this.originNotify) {
                    console.log('down', str);
                    _this.originNotify(_this.device, {
                        data: str || '',
                        down: true,
                        time: (0, dayjs_1.default)().format('YYYY-MM-DD hh:mm:ss')
                    });
                }
                if (_this.isLastPackage) {
                    // 兼容1.0
                    if (type === 'auth' && _this.authed) {
                        _this.device.setAuth(true);
                        _this.authed(_this.device);
                    }
                    if (_this.sent) {
                        _this.sent(_this.device, type);
                    }
                }
            },
            fail: function (res) {
                console.log({ res: res });
                if (res.errCode === 10000) {
                    _this.closeBluetoothAdapter();
                    _this.searchBlue();
                    // Toast.fail('蓝牙已经断开请重试')
                    if (_this.fail) {
                        _this.fail(error_1.default.sendError, res);
                    }
                    else {
                        _this.toast('发送数据时存在异常，蓝牙已经断开', 'error');
                    }
                }
            },
        });
    };
    // 发送数据
    BlueService.prototype.send = function (str, type) {
        if (type === void 0) { type = 'none'; }
        var buffer = (0, util_1.toBlueBufferData)(str);
        this.setIsLastPackage(true);
        this.writeToBlue(buffer[0], type, str);
    };
    BlueService.prototype.sendChunk = function (str, type, len) {
        var _this = this;
        if (type === void 0) { type = 'none'; }
        if (len === void 0) { len = 30; }
        var buffers = (0, util_1.toBlueBufferData)(str, len);
        var packageCount = buffers.length;
        if (packageCount > 1 && this.isLastPackage) {
            this.setIsLastPackage(false);
        }
        var cur = 1;
        buffers.forEach(function (buffer) {
            if (cur >= packageCount) {
                _this.setIsLastPackage(true);
            }
            _this.writeToBlue(buffer, type, str.substring((cur - 1) * len * 2, cur * len * 2));
            cur++;
        });
    };
    BlueService.prototype.handleNextCb = function (str, type, detail) {
        if (type === void 0) { type = 'send'; }
        if (detail === void 0) { detail = { type: 'none', status: false }; }
        if (type === 'noSend') {
            if (!this.device) {
                if (this.fail) {
                    this.fail(error_1.default.notFoundDeviceError, '未找到需要连接的设备');
                }
                else {
                    this.toast('未找到需要连接的设备', 'error');
                }
                return;
            }
            var isNotify = false;
            switch (detail.type) {
                case Methods_1.default.AUTH: {
                    this.device.setAuth(detail.status || false);
                    if (this.updated) {
                        this.updated(this.device);
                    }
                    if (this.authed) {
                        this.authed(this.device);
                    }
                    break;
                }
                case Methods_1.default.ON: {
                    isNotify = this.device && this.device.state === 0;
                    this.device.setState(1);
                    break;
                }
                case Methods_1.default.OFF: {
                    isNotify = this.device && this.device.state === 1;
                    this.device.setState(0);
                    break;
                }
                // case 'face-ok': {
                //     if (this.notify) {
                //         this.notify(this.device, detail)
                //     }
                //     if (BlueService.callback) BlueService.callback({
                //         type: 'refresh',
                //         detail: detail.value
                //     })
                //     break;
                // }
                default: {
                    isNotify = true;
                }
            }
            if (isNotify && this.notify) {
                this.notify(this.device, detail);
            }
            return;
        }
        if (type === 'send') {
            this.sendChunk(str, detail.type);
        }
        else {
            this.send(str, detail.type);
        }
    };
    BlueService.prototype.handleProcessCb = function (process, detail) {
        if (!this.device) {
            if (this.fail) {
                this.fail(error_1.default.notFoundDeviceError, '未找到需要连接的设备');
            }
            else {
                this.toast('未找到需要连接的设备', 'error');
            }
            return;
        }
        this.device.setProcess(process);
        if (this.process) {
            this.process(this.device, process, detail);
        }
    };
    BlueService.prototype.handleFailCb = function (code, message) {
        if (this.device) {
            this.device.setProcess(false);
        }
        if (this.fail) {
            this.fail(code, message);
        }
        else {
            this.toast(message, 'error');
        }
    };
    BlueService.prototype.handle = function (method, detail) {
        var _this = this;
        if (method === void 0) { method = 'turnOn'; }
        if (detail === void 0) { detail = {}; }
        if (!this.device || !this.device.version) {
            if (this.fail) {
                this.fail(error_1.default.notFoundVersionError, '未找到蓝牙版本');
            }
            else {
                this.toast('未找到蓝牙版本', 'error');
            }
            return;
        }
        if (!this.device.connected) {
            console.log('蓝牙未连接！', this.device);
            if (this.fail) {
                this.fail(error_1.default.notConnected, '蓝牙未连接');
            }
            else {
                this.toast('蓝牙连接失败！', 'error');
            }
            return;
        }
        if (!this.device.auth) {
            if (this.fail) {
                this.fail(error_1.default.authFailError, '蓝牙认证失败!');
            }
            else {
                this.toast('蓝牙认证失败!', 'error');
            }
            return;
        }
        if (!this.device.write) {
            if (this.fail) {
                this.fail(error_1.default.cantWriteableError, '蓝牙不可写入!');
            }
            else {
                this.toast('蓝牙不可写入!', 'error');
            }
            return;
        }
        // if ((!this.config.isContinue) && this.device.getProcess()) {
        //     if (this.fail) {
        //         this.fail(errorCode.processOnError, '设备正在接收处理，10秒之后再试!')
        //     } else {
        //         this.toast('设备正在接收处理，10秒之后再试!', 'error')
        //     }
        //     return;
        // }
        if (!this.device.getProcess()) {
            this.device.setProcess(true);
            if (this.process) {
                this.process(this.device, true, {
                    type: method,
                    status: true,
                    process: 2,
                    message: '初始化'
                });
            }
        }
        try {
            /**
             * @param {CommonVersion} v
             */
            var v = new (versionToService[this.device.version])({
                fail: function (code, message) {
                    _this.handleFailCb(code, message);
                },
                next: function (str, type, d) {
                    if (type === void 0) { type = 'send'; }
                    if (d === void 0) { d = {
                        type: 'none',
                        status: false
                    }; }
                    _this.handleNextCb(str, type, d);
                },
                process: function (process, d) {
                    _this.handleProcessCb(process, d);
                }
            }, this.config);
            if (!detail.ecsNo) {
                detail.ecsNo = this.device.ecsNo;
            }
            v.handle(method, detail);
        }
        catch (error) {
            this.device.setProcess(false);
            if (this.device.getProcess()) {
                if (this.process) {
                    this.process(this.device, false, {
                        type: method,
                        status: false,
                        process: 1,
                        message: '处理时发生错误'
                    });
                }
            }
            console.log({ error: error });
            if (this.fail) {
                this.fail(error_1.default.handleError, '处理时发生错误!');
            }
            else {
                this.toast('处理时发生错误!', 'error');
            }
        }
    };
    return BlueService;
}());
exports.BlueService = BlueService;
