"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.floatToHex32 = exports.swapEndian = exports.strSplit = exports.sprintf = exports.pregExpire = exports.isLeapYear = exports.pregMobile = exports.formatDate = exports.toBlueBufferData = exports.decodeFromGb2312 = exports.encodeToGb2312 = exports.ab2hex = exports.inArray = exports.formatTime = void 0;
var gb2312_1 = require("./gb2312");
var formatTime = function (date) {
    var year = date.getFullYear();
    var month = date.getMonth() + 1;
    var day = date.getDate();
    var hour = date.getHours();
    var minute = date.getMinutes();
    var second = date.getSeconds();
    return ([year, month, day].map(formatNumber).join('/') +
        ' ' +
        [hour, minute, second].map(formatNumber).join(':'));
};
exports.formatTime = formatTime;
var formatNumber = function (n) {
    var s = n.toString();
    return s[1] ? s : '0' + s;
};
var inArray = function (arr, key, val) {
    for (var i = 0; i < arr.length; i++) {
        if (arr[i][key] === val) {
            return i;
        }
    }
    return -1;
};
exports.inArray = inArray;
// ArrayBuffer转16进度字符串示例
var ab2hex = function (buffer) {
    var hexArr = Array.prototype.map.call(new Uint8Array(buffer), function (bit) { return ('00' + bit.toString(16)).slice(-2); });
    return hexArr.join('');
};
exports.ab2hex = ab2hex;
var encodeToGb2312 = function (str, len) {
    if (len === void 0) { len = 0; }
    var strOut = "";
    // cha
    for (var i = 0; i < str.length; i++) {
        var c = str.charAt(i);
        var code = str.charCodeAt(i);
        if (c === " ") {
            strOut += "+";
        }
        else if (code >= 19968 && code <= 40869) {
            var index = (code - 19968) * 4;
            strOut += gb2312_1.GB2312_CI.substring(index, index + 2) + gb2312_1.GB2312_CI.substring(index + 2, index + 4);
        }
        else {
            strOut += (str.charCodeAt(i) + 41856).toString(16).toUpperCase();
        }
    }
    return len ? strOut.padEnd(20, '0') : strOut;
};
exports.encodeToGb2312 = encodeToGb2312;
var decodeFromGb2312 = function (str) {
    var strOut = '';
    for (var i = 0; i < str.length; i++) {
        var c = str.charAt(i);
        // +是空格
        if (c === '+') {
            strOut += ' ';
        }
        // a,b,c,1,2等，非%开头的，直接返回本身
        else if (c !== '%') {
            strOut += c;
        }
        // %开头
        else {
            i++;
            var nextC = str.charAt(i);
            // 数字，则不是汉字
            if (!isNaN(parseInt(nextC, 10))) {
                i++;
                strOut += decodeURIComponent(c + nextC + str.charAt(i));
            }
            else {
                try {
                    var code = str.substring(i, i + 2) + str.substring(i + 3, i + 5);
                    i = i + 4;
                    var index = gb2312_1.GB2312_CI.indexOf(code, 0);
                    while (index !== -1) {
                        if (index % 4 === 0) {
                            strOut += String.fromCharCode(index / 4 + 19968);
                            break;
                        }
                        index = gb2312_1.GB2312_CI.indexOf(code, index + 1);
                    }
                }
                catch (e) {
                    console.error(e);
                }
            }
        }
    }
    return strOut;
};
exports.decodeFromGb2312 = decodeFromGb2312;
var bufferArray = function (arr) {
    var commandArray = new Uint8Array(arr);
    var buffer = new ArrayBuffer(commandArray.length);
    var dataView = new DataView(buffer);
    for (var j = 0; j < commandArray.length; j++) {
        if (commandArray[j] !== undefined) {
            dataView.setUint8(j, commandArray[j]);
        }
    }
    return buffer;
};
var toBlueBufferData = function (str, chunkLen) {
    if (chunkLen === void 0) { chunkLen = 0; }
    var data = str.match(/[\da-f]{2}/gi);
    if (!data) {
        return [];
    }
    var temp = data.map(function (h) { return parseInt(h, 16); });
    if (chunkLen === 0) {
        return [bufferArray(temp)];
    }
    else {
        var buffers = [];
        while (temp.length > 0) {
            var arr = temp.splice(0, chunkLen);
            buffers.push(bufferArray(arr));
        }
        return buffers;
    }
};
exports.toBlueBufferData = toBlueBufferData;
function formatDate(d) {
    var date = new Date(d);
    return "".concat(date.getFullYear(), "-").concat(String(date.getMonth() + 1).padStart(2, '0'), "-").concat(String(date.getDate()).padStart(2, '0'));
}
exports.formatDate = formatDate;
var pregMobile = function (mobile) {
    return /^1[3-9]\d{9}$/.test(mobile);
};
exports.pregMobile = pregMobile;
var isLeapYear = function (year) {
    return ((year % 4) === 0 && (year % 100) !== 0) || (year % 400) === 0;
};
exports.isLeapYear = isLeapYear;
var pregExpire = function (expire) {
    var res = expire.match(/^(20[\d]{2})-(0[1-9]|[1-9]|1[0-2])-(0[1-9]|[1-9]|[12][\d]|3[0-1])$/);
    if (!res) {
        return false;
    }
    var year = parseInt(res[1], 10);
    var month = parseInt(res[2], 10);
    var day = parseInt(res[3], 10);
    switch (month) {
        case 2: {
            if ((0, exports.isLeapYear)(year)) {
                return day <= 29;
            }
            return day <= 28;
        }
        case 4:
        case 6:
        case 9:
        case 11: {
            return day <= 30;
        }
        default: {
            return day <= 31;
        }
    }
};
exports.pregExpire = pregExpire;
function str_repeat(i, m) {
    var o = [];
    for (o; m > 0; o[--m] = i)
        ;
    return o.join('');
}
var sprintf = function () {
    var params = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        params[_i] = arguments[_i];
    }
    var i = 0;
    var a;
    var f = params[i++];
    var m;
    var p;
    var c;
    var x;
    var s = '';
    var o = [];
    while (f) {
        // tslint:disable-next-line: no-conditional-assignment
        if ((m = /^[^\x25]+/.exec(f))) {
            o.push(m[0]);
            // tslint:disable-next-line: no-conditional-assignment
        }
        else if ((m = /^\x25{2}/.exec(f))) {
            o.push('%');
            // tslint:disable-next-line: no-conditional-assignment
        }
        else if ((m = /^\x25(?:(\d+)\$)?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-fosuxX])/.exec(f))) {
            // @ts-ignore
            // tslint:disable-next-line: no-conditional-assignment
            if (((a = params[m[1] || i++]) == null)) {
                throw Error('Too few params.');
            }
            if (/[^s]/.test(m[7]) && (typeof (a) !== 'number')) {
                throw Error('Expecting number but found ' + typeof (a));
            }
            switch (m[7]) {
                case 'b':
                    a = a.toString(2);
                    break;
                case 'c':
                    a = String.fromCharCode(a);
                    break;
                case 'd':
                    a = parseInt(a, 10);
                    break;
                case 'e':
                    a = m[6] ? a.toExponential(m[6]) : a.toExponential();
                    break;
                case 'f': // @ts-ignore
                    a = m[6] ? parseFloat(a).toFixed(m[6]) : parseFloat(a);
                    break;
                case 'o':
                    a = a.toString(8);
                    break;
                case 's': // @ts-ignore
                    // tslint:disable-next-line: no-conditional-assignment
                    a = ((a = String(a)) && m[6] ? String(a).substring(0, m[6]) : String(a));
                    break;
                case 'u':
                    a = Math.abs(a);
                    break;
                case 'x':
                    a = a.toString(16);
                    break;
                case 'X':
                    a = a.toString(16).toUpperCase();
                    break;
            }
            a = (/[def]/.test(m[7]) && m[2] && a >= 0 ? '+' + a : a);
            c = m[3] ? String(m[3]) === '0' ? '0' : m[3].charAt(1) : ' ';
            // @ts-ignore
            x = m[5] - String(a).length - s.length;
            p = m[5] ? str_repeat(c, x) : '';
            o.push(s + (m[4] ? a + p : p + a));
        }
        else {
            throw Error('Huh ?!');
        }
        f = f.substring(m[0].length);
    }
    return o.join('');
};
exports.sprintf = sprintf;
var strSplit = function (str, len) {
    if (len === void 0) { len = 1; }
    var strArr = [];
    var l = str.length;
    for (var i = 0; i < l; i += len) {
        strArr.push(str.substring(i, i + len));
    }
    return strArr;
};
exports.strSplit = strSplit;
// 高低位反转
var swapEndian = function (v) {
    return (v.match(/[\da-f]{2}/gi) || []).reverse().join("");
};
exports.swapEndian = swapEndian;
// 浮点数转32位二进制（ieee-754）
var floatToHex32 = function (value) {
    // 检查输入是否为有效的浮点数
    if (isNaN(value) || !isFinite(value)) {
        throw new Error("Invalid input, please provide a valid floating-point number.");
    }
    // 使用DataView来处理浮点数的二进制表示
    var buffer = new ArrayBuffer(4); // 32位浮点数占用4个字节
    var view = new DataView(buffer);
    view.setFloat32(0, value, false); // 第三个参数表示字节序，false表示使用小端字节序
    // 获取每个字节的十六进制表示，并拼接起来
    var hexString = Array.from(new Uint8Array(buffer))
        .map(function (byte) { return byte.toString(16).padStart(2, '0'); })
        .reverse() // 需要反转字节序 (小端)
        .join('');
    return hexString;
};
exports.floatToHex32 = floatToHex32;
